﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopDataModels.Enums;
using BlacksmithWorkshopDataModels.Models;
using System.Xml.Linq;

namespace BlacksmithWorkshopFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public int ManufactureId {get; private set; }
        public int Count { get; private set; }
        public double Sum { get; private set; }
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateImplement { get; private set; }
        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                ManufactureId = model.ManufactureId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }
        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            DateTime? dateImplement = null;
            if (element.Element("DateImplement")!.Value != "")
            {
                dateImplement = Convert.ToDateTime(element.Element("DateImplement")!.Value);
            }
            return new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ManufactureId = Convert.ToInt32(element.Element("ManufactureId")!.Value),
                Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                Status = (OrderStatus)Convert.ToInt32(element.Element("Status")!.Value),
                DateCreate = Convert.ToDateTime(element.Element("DateCreate")!.Value),
                DateImplement = dateImplement
            };
        }
        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            if (model.DateImplement != null)
            {
                DateImplement = model.DateImplement;
            }
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            ManufactureId = ManufactureId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
        public XElement GetXElement => new
        (
        "Order",
        new XAttribute("Id", Id),
        new XElement("ManufactureId", ManufactureId),
        new XElement("Count", Count),
        new XElement("Sum", Sum),
        new XElement("Status", (int)Status),
        new XElement("DateCreate", DateCreate),
        new XElement("DateImplement", DateImplement)
        );
    }
}