﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopListImplement.Models;

namespace BlacksmithWorkshopListImplement.Implements
{
    public class ManufactureStorage : IManufactureStorage
    {
        private readonly DataListSingleton _source;
        public ManufactureStorage()
        {
            _source = DataListSingleton.GetInstance();
        }
        public List<ManufactureViewModel> GetFullList()
        {
            var result = new List<ManufactureViewModel>();
            foreach (var manufacture in _source.Manufactures)
            {
                result.Add(manufacture.GetViewModel);
            }
            return result;
        }
        public List<ManufactureViewModel> GetFilteredList(ManufactureSearchModel model)
        {
            var result = new List<ManufactureViewModel>();
            if (string.IsNullOrEmpty(model.ManufactureName))
            {
                return result;
            }
            foreach (var manufacture in _source.Manufactures)
            {
                if (manufacture.ManufactureName.Contains(model.ManufactureName))
                {
                    result.Add(manufacture.GetViewModel);
                }
            }
            return result;
        }
        public ManufactureViewModel? GetElement(ManufactureSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ManufactureName) && !model.Id.HasValue)
            {
                return null;
            }
            foreach (var manufacture in _source.Manufactures)
            {
                if (!string.IsNullOrEmpty(model.ManufactureName) && manufacture.ManufactureName == model.ManufactureName || model.Id.HasValue && manufacture.Id == model.Id)
                {
                    return manufacture.GetViewModel;
                }
            }
            return null;
        }
        public ManufactureViewModel? Insert(ManufactureBindingModel model)
        {
            model.Id = 1;
            foreach (var manufacture in _source.Manufactures)
            {
                if (model.Id <= manufacture.Id)
                {
                    model.Id = manufacture.Id + 1;
                }
            }
            var newManufacture = Manufacture.Create(model);
            if (newManufacture == null)
            {
                return null;
            }
            _source.Manufactures.Add(newManufacture);
            return newManufacture.GetViewModel;
        }
        public ManufactureViewModel? Update(ManufactureBindingModel model)
        {
            foreach (var manufacture in _source.Manufactures)
            {
                if (manufacture.Id == model.Id)
                {
                    manufacture.Update(model);
                    return manufacture.GetViewModel;
                }
            }
            return null;
        }
        public ManufactureViewModel? Delete(ManufactureBindingModel model)
        {
            for (int i = 0; i < _source.Manufactures.Count; i++)
            {
                if (_source.Manufactures[i].Id == model.Id)
                {
                    var element = _source.Manufactures[i];
                    _source.Manufactures.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
    }
}