﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopListImplement.Models;

namespace BlacksmithWorkshopListImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataListSingleton _source;
        public OrderStorage()
        {
            _source = DataListSingleton.GetInstance();
        }
        public List<OrderViewModel> GetFullList()
        {
            var result = new List<OrderViewModel>();
            foreach (var order in _source.Orders)
            {
                OrderViewModel vm = order.GetViewModel;
                var manufacture = _source.Manufactures.Find(x => x.Id == order.ManufactureId);
                vm.ManufactureName = manufacture?.ManufactureName ?? string.Empty;
                result.Add(vm);
            }
            return result;
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            var result = new List<OrderViewModel>();
            if (!model.Id.HasValue && (!model.DateFrom.HasValue || !model.DateTo.HasValue))
            {
                return result;
            }
            if (model.Id.HasValue)//сначала ищем по Id
            {
                foreach (var order in _source.Orders)
                {
                    if (model.Id == order.Id)
                    {
                        OrderViewModel vm = order.GetViewModel;
                        var manufacture = _source.Manufactures.Find(x => x.Id == order.ManufactureId);
                        vm.ManufactureName = manufacture?.ManufactureName ?? string.Empty;
                        result.Add(vm);
                    }
                }
            }
            else if (model.DateFrom.HasValue && model.DateTo.HasValue)//если у модели нет Id, ищем по датам
            {
                foreach (var order in _source.Orders)
                {
                    if (order.DateCreate >= model.DateFrom && order.DateCreate <= model.DateTo)
                    {
                        OrderViewModel vm = order.GetViewModel;
                        var manufacture = _source.Manufactures.Find(x => x.Id == order.ManufactureId);
                        vm.ManufactureName = manufacture?.ManufactureName ?? string.Empty;
                        result.Add(vm);
                    }
                }
            }
            return result;
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            foreach (var order in _source.Orders)
            {
                if (model.Id.HasValue && order.Id == model.Id)
                {
                    return order.GetViewModel;
                }
            }
            return null;
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = 1;
            foreach (var order in _source.Orders)
            {
                if (model.Id <= order.Id)
                {
                    model.Id = order.Id + 1;
                }
            }
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            _source.Orders.Add(newOrder);
            return newOrder.GetViewModel;
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            foreach (var order in _source.Orders)
            {
                if (order.Id == model.Id)
                {
                    order.Update(model);
                    return order.GetViewModel;
                }
            }
            return null;
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            for (int i = 0; i < _source.Orders.Count; i++)
            {
                if (_source.Orders[i].Id == model.Id)
                {
                    var element = _source.Orders[i];
                    _source.Orders.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
    }
}